from django.db import models
from django.db.models import Index
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_model.models import BaseUuidModel
from edc_visit_schedule.model_mixins import (
    VisitCodeFieldsModelMixin,
    VisitScheduleFieldsModelMixin,
)


class CrfStatus(
    NonUniqueSubjectIdentifierFieldMixin,
    VisitScheduleFieldsModelMixin,
    VisitCodeFieldsModelMixin,
    BaseUuidModel,
):
    label_lower = models.CharField(max_length=150, null=True)

    class Meta(BaseUuidModel.Meta, NonUniqueSubjectIdentifierFieldMixin.Meta):
        verbose_name = "CRF Status"
        verbose_name_plural = "CRF Status"
        indexes = (
            BaseUuidModel.Meta.indexes
            + NonUniqueSubjectIdentifierFieldMixin.Meta.indexes
            + [
                Index(
                    fields=[
                        "schedule_name",
                        "visit_schedule_name",
                        "visit_code",
                        "visit_code_sequence",
                    ]
                )
            ]
        )



# Generated by Django 4.2.7 on 2023-12-04 22:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_crf", "0006_remove_crfstatus_edc_crf_crf_modifie_6152c9_idx_and_more"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="crfstatus",
            index=models.Index(
                fields=["subject_identifier"], name="edc_crf_crf_subject_e09e96_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="crfstatus",
            index=models.Index(
                fields=[
                    "schedule_name",
                    "visit_schedule_name",
                    "visit_code",
                    "visit_code_sequence",
                ],
                name="edc_crf_crf_schedul_f8d84b_idx",
            ),
        ),
    ]



from django import forms
from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _
from edc_model import model_exists_or_raise


def raise_if_crf_does_not_exist(subject_visit, model: str) -> None:
    model_cls = django_apps.get_model(model)
    try:
        model_exists_or_raise(subject_visit=subject_visit, model_cls=model_cls)
    except ObjectDoesNotExist:
        complete = _("Complete")
        crf_first = _("CRF first")
        verbose_name = model_cls._meta.verbose_name
        err_message = format_lazy(
            "{complete} {verbose_name} {crf_first}",
            complete=complete,
            crf_first=crf_first,
            verbose_name=verbose_name,
        )
        raise forms.ValidationError(err_message)



from django.conf import settings
from django.db import models
from edc_consent.model_mixins import RequiresConsentFieldsModelMixin
from edc_metadata.model_mixins.updates import UpdatesCrfMetadataModelMixin
from edc_offstudy.model_mixins import OffstudyCrfModelMixin
from edc_visit_tracking.model_mixins import (
    PreviousVisitModelMixin,
    VisitTrackingCrfModelMixin,
)


class CrfNoManagerModelMixin(
    VisitTrackingCrfModelMixin,
    RequiresConsentFieldsModelMixin,
    PreviousVisitModelMixin,
    UpdatesCrfMetadataModelMixin,
    OffstudyCrfModelMixin,
    models.Model,
):
    """Modelmixin for all scheduled CRF models"""

    def natural_key(self) -> tuple:
        return self.related_visit.natural_key()

    natural_key.dependencies = [
        settings.SUBJECT_VISIT_MODEL,
        "sites.Site",
        "edc_appointment.appointment",
    ]

    class Meta:
        abstract = True
        indexes = [
            models.Index(fields=["subject_visit", "site"]),
            models.Index(fields=["subject_visit", "report_datetime"]),
        ]
        default_permissions = ("add", "change", "delete", "view", "export", "import")



from __future__ import annotations


class CrfStatusModelAdminMixin:
    def get_list_display(self, request) -> tuple[str, ...]:
        list_display = super().get_list_display(request)
        if "crf_status" not in list_display:
            list_display = list_display + ("crf_status",)
        return list_display

    def get_list_filter(self, request) -> tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        if "crf_status" not in list_filter:
            list_filter = ("crf_status",) + list_filter
        return list_filter



from __future__ import annotations

from typing import TYPE_CHECKING

from django import forms
from django.core.exceptions import ObjectDoesNotExist
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _

if TYPE_CHECKING:
    from .crf_model_form_mixin import CrfModelFormMixin

    class MyForm(CrfModelFormMixin, forms.ModelForm):
        pass


class CrfSingletonModelFormMixin:
    def clean(self) -> dict:
        cleaned_data = super().clean()
        self.raise_if_singleton_exists()
        return cleaned_data

    def raise_if_singleton_exists(self: MyForm) -> None:
        """Raise if singleton model instance exists at another
        timepoint.
        """
        if not self.instance.id:
            opts = {
                f"{self.related_visit_model_attr}__subject_identifier": self.subject_identifier
            }
            try:
                obj = self._meta.model.objects.get(**opts)
            except ObjectDoesNotExist:
                pass
            else:
                msg_string = _("Invalid. This form has already been submitted. See")
                visit_code = f"{obj.visit_code}.{obj.visit_code_sequence}"
                error_msg = format_lazy(
                    "{msg_string} {visit_code}", msg_string=msg_string, visit_code=visit_code
                )
                raise forms.ValidationError(error_msg)



from django.db import models
from edc_model.models import BaseUuidModel
from edc_utils import get_utcnow

from edc_crf.model_mixins import CrfModelMixin, CrfStatusModelMixin


class Crf(CrfModelMixin, CrfStatusModelMixin, BaseUuidModel):
    f1 = models.CharField(max_length=50, null=True, blank=True)

    f2 = models.CharField(max_length=50, null=True, blank=True)

    f3 = models.CharField(max_length=50, null=True, blank=True)

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        pass


class CrfReportDatetimeNotRequired(CrfModelMixin, CrfStatusModelMixin, BaseUuidModel):
    report_datetime = models.DateTimeField(default=get_utcnow, null=True, blank=True)

    f1 = models.CharField(max_length=50, null=True, blank=True)

    f2 = models.CharField(max_length=50, null=True, blank=True)

    f3 = models.CharField(max_length=50, null=True, blank=True)

    def update_reference_on_save(self):
        pass

    def metadata_update(self):
        pass

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        pass



from __future__ import annotations

from edc_consent.modelform_mixins import RequiresConsentModelFormMixin
from edc_form_validators import FormValidatorMixin
from edc_model_form.mixins import BaseModelFormMixin
from edc_offstudy.modelform_mixins import OffstudyCrfModelFormMixin
from edc_sites.forms import SiteModelFormMixin
from edc_visit_schedule.modelform_mixins import VisitScheduleCrfModelFormMixin
from edc_visit_tracking.modelform_mixins import VisitTrackingCrfModelFormMixin


class CrfModelFormMixin(
    SiteModelFormMixin,
    RequiresConsentModelFormMixin,
    VisitScheduleCrfModelFormMixin,
    VisitTrackingCrfModelFormMixin,
    OffstudyCrfModelFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
):
    """A modelform mixin for all CRFs.

    * Checks for the consent relative to report datetime
      and this schedule;
    * is participant on/off schedule relative to report
      datetime and this schedule;
    * validates subject_visit report datetime;
    * is participant offstudy relative to report datetime.
    """

    report_datetime_field_attr = "report_datetime"
